// Xmas Contest 2014
// Output Checker for "A+B Problem"
// Author: hos <hos@hos.ac>

// program        ::= line*
// line           ::= multiplication | division | opening | closing
// multiplication ::= variable "=" factor "*" factor
// division       ::= variable "=" factor "/" factor
// opening        ::= "if" "(" factor ")" "{"
// closing        ::= "}"
// factor         ::= variable | constant
// variable       ::= "A" | "B" | "C" | "D" | ... | "Z"
// constant       ::= digit*
// digit          ::= "0" | ... | "9"

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <vector>

using std::string;
using std::vector;

#ifndef UINT32_MAX
#define UINT32_MAX (4294967295U)
#endif
#ifndef UINT64_MAX
#define UINT64_MAX (18446744073709551615ULL)
#endif

enum FactorType {
  kUnknownFactor,
  kVariable,
  kConstant,
};
enum LineType {
  kUnknownLine,
  kMultiplication,
  kDivision,
  kOpening,
  kClosing,
};

struct Factor {
  FactorType type;
  uint32_t value;
};
struct Line {
  LineType type;
  vector<Factor> factors;
};

const int kMaxNumberOfLines = 999;
const int kMaxWidth = 80;
const int kBufferSize = kMaxWidth + 10;
const string kEndOfLineCharacters = "\n\r";
const string kWhiteSpaceCharacters = "\t ";
const string kTokenCharacters = "=*/if(){}";

const int kNumVariables = 26;
const uint32_t kMinTestValue = 1;
const uint32_t kMaxTestValue = 100;

// Show message to the contestant, tell CMS that the score is zero, and exit.
void ReportError(const char *message) {
  fprintf(stderr, "%s\n", message);
  printf("0.00\n");
  exit(0);
}

// Return true if a factor successfully parsed.
bool ParseFactor(const string &factor_raw, Factor *factor) {
  factor->type = kUnknownFactor;
  factor->value = 0;
  if (factor_raw.size() == 1 &&
      ('A' <= factor_raw[0] && factor_raw[0] <= 'Z')) {
    factor->type = kVariable;
    factor->value = factor_raw[0] - 'A';
  } else {
    factor->type = kConstant;
    uint64_t value = 0;
    for (const char c : factor_raw) {
      if (!('0' <= c && c <= '9')) {
        return false;
      }
      value = value * 10 + (c - '0');
      if (value > UINT32_MAX) {
        return false;
      }
    }
    if (!(0 <= value && value <= UINT32_MAX)) {
      return false;
    }
    factor->value = value;
  }
  return true;
}

// Return true if a line is successfully parsed.
bool ParseLine(const string &line_raw, Line *line) {
  line->type = kUnknownLine;
  line->factors.clear();
  if (line_raw.find('=') != string::npos) {
    const char operation = (line_raw.find('*') != string::npos) ? '*' : '/';
    line->type = (operation == '*') ? kMultiplication : kDivision;
    const size_t pos1 = line_raw.find('=');
    if (pos1 == string::npos) {
      return false;
    }
    const size_t pos2 = line_raw.find(operation, pos1 + 1);
    if (pos2 == string::npos) {
      return false;
    }
    const string factors_raw[] = {line_raw.substr(0, pos1),
                                  line_raw.substr(pos1 + 1, pos2 - (pos1 + 1)),
                                  line_raw.substr(pos2 + 1)};
    if (line_raw !=
        factors_raw[0] + "=" + factors_raw[1] + operation + factors_raw[2]) {
      return false;
    }
    for (const string &factor_raw : factors_raw) {
      Factor factor;
      if (!ParseFactor(factor_raw, &factor)) {
        return false;
      }
      line->factors.push_back(factor);
    }
    if (line->factors[0].type != FactorType::kVariable) {
      return false;
    }
  } else if (line_raw.find('{') != string::npos) {
    line->type = kOpening;
    const size_t pos1 = line_raw.find('(');
    if (pos1 == string::npos) {
      return false;
    }
    const size_t pos2 = line_raw.find(')', pos1 + 1);
    if (pos2 == string::npos) {
      return false;
    }
    const string factor_raw = line_raw.substr(pos1 + 1, pos2 - (pos1 + 1));
    if (line_raw != "if(" + factor_raw + "){") {
      return false;
    }
    Factor factor;
    if (!ParseFactor(factor_raw, &factor)) {
      return false;
    }
    line->factors.push_back(factor);
  } else if (line_raw.find('}') != string::npos) {
    line->type = kClosing;
    if (line_raw != "}") {
      return false;
    }
  } else {
    // Unexpected line type.
    return false;
  }
  return true;
}

// Look up the current value of a variable in the memory,
// or return the value of a constant.
uint32_t Evaluate(const Factor &factor, const vector<uint32_t> &memory) {
  switch (factor.type) {
    case kVariable:
      return memory[factor.value];
    case kConstant:
      return factor.value;
    default:
      fprintf(stderr, "[SYSTEM ERROR] Please report this to admin.\n");
      exit(1);
  }
}

// Execute the program.
// Set a flag and return if any mistake is found.
void ExecuteProgram(const vector<Line> &lines, uint32_t a, uint32_t b,
                    bool *flag_division_by_zero, bool *flag_wrong_answer) {
  vector<uint32_t> memory(kNumVariables, 0);
  memory[0] = a;
  memory[1] = b;
  int depth = 0, depth_running = 0;
  for (const Line &line : lines) {
    switch (line.type) {
      case kMultiplication:
        if (depth == depth_running) {
          const uint32_t operand1 = Evaluate(line.factors[1], memory);
          const uint32_t operand2 = Evaluate(line.factors[2], memory);
          memory[line.factors[0].value] = operand1 * operand2;
        }
        break;
      case kDivision:
        if (depth == depth_running) {
          const uint32_t operand1 = Evaluate(line.factors[1], memory);
          const uint32_t operand2 = Evaluate(line.factors[2], memory);
          if (operand2 == 0) {
            *flag_division_by_zero = true;
            return;
          }
          memory[line.factors[0].value] = operand1 / operand2;
        }
        break;
      case kOpening:
        if (depth == depth_running) {
          const uint32_t condition = Evaluate(line.factors[0], memory);
          if (condition != 0) {
            ++depth_running;
          }
        }
        ++depth;
        break;
      case kClosing:
        if (depth == depth_running) {
          --depth_running;
        }
        --depth;
        break;
      default:
        fprintf(stderr,
                "[SYSTEM ERROR] Please report this to the administrator.\n");
        exit(1);
    }
  }
  if (memory[2] != a + b) {
    *flag_wrong_answer = true;
    return;
  }
}

int main(int argc, char **argv) {
  if (argc <= 3) {
    fprintf(stderr,
            "[SYSTEM ERROR] Usage: checker [input] [answer] [output]\n");
    exit(1);
  }

  // Read the contestant's output.
  FILE *output = fopen(argv[3], "r");
  if (!output) {
    fprintf(stderr, "[SYSTEM ERROR] Cannot open the output file.\n");
    exit(1);
  }
  vector<string> program;
  for (int i = 0; ; ++i) {
    char buffer[kBufferSize];
    if (!fgets(buffer, kBufferSize, output)) {
      break;
    }
    if (i >= kMaxNumberOfLines) {
      fclose(output);
      ReportError("[PARSE ERROR] The number of lines exceeds the limit.");
    }
    int width = 0;
    string line_raw = "";
    for (int j = 0; buffer[j] != '\0'; ++j) {
      const char c = buffer[j];
      if (kEndOfLineCharacters.find(c) != string::npos) {
        break;
      } else if (kWhiteSpaceCharacters.find(c) != string::npos) {
        ++width;
      } else if (kTokenCharacters.find(c) != string::npos ||
                 ('0' <= c && c <= '9') || ('A' <= c && c <= 'Z')) {
        ++width;
        line_raw += c;
      } else {
        fclose(output);
        ReportError("[PARSE ERROR] Unexpected character.");
      }
    }
    if (width > kMaxWidth) {
      fclose(output);
      ReportError("[PARSE ERROR] The width of a line exceeds the limit.");
    }
    program.push_back(line_raw);
  }
  fclose(output);

  // Parse the program.
  vector<Line> lines;
  for (const string &line_raw : program) {
    Line line;
    if (!ParseLine(line_raw, &line)) {
      ReportError("[PARSE ERROR] Fail to parse a line.");
    }
    lines.push_back(line);
  }
  int depth = 0;
  for (const Line &line : lines) {
    switch (line.type) {
      case kMultiplication:
        break;
      case kDivision:
        break;
      case kOpening:
        ++depth;
        break;
      case kClosing:
        --depth;
        if (depth < 0) {
          ReportError("[PARSE ERROR] Braces are not properly matching.");
        }
        break;
      default:
        fprintf(stderr,
                "[SYSTEM ERROR] Please report this to the administrator.\n");
        exit(1);
    }
  }
  if (depth != 0) {
    ReportError("[PARSE ERROR] Braces are not properly matching.");
  }

  // Execute and judge the program.
  bool flag_division_by_zero = false;
  bool flag_wrong_answer = false;
  for (uint32_t a = kMinTestValue; a <= kMaxTestValue; ++a) {
    for (uint32_t b = kMinTestValue; b <= kMaxTestValue; ++b) {
      ExecuteProgram(lines, a, b, &flag_division_by_zero, &flag_wrong_answer);
    }
  }
  if (flag_division_by_zero) {
    ReportError("[RUNTIME ERROR] Division by zero.");
  }
  if (flag_wrong_answer) {
    ReportError("[WRONG ANSWER] The program does not correctly compute A + B.");
  }

  // Scoring
  const int num_lines = lines.size();
  fprintf(stderr, "[ACCEPTED] Correct! (%d lines)\n", num_lines);
  printf("%.2f\n", (100 - num_lines / 10) / 100.0);

  return 0;
}
